package info.kofler.fahrenheit

import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.view.Gravity
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import info.kofler.fahrenheit.databinding.ActivityMainBinding
import kotlin.math.round

class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)

        binding.editFahrenheit.addTextChangedListener(object :
            TextWatcher {
            override fun afterTextChanged(s: Editable?) { }

            override fun beforeTextChanged(
                s: CharSequence?, start: Int, count: Int,
                after: Int) { }

            override fun onTextChanged(s: CharSequence?,
                                       start: Int,
                                       before: Int,
                                       count: Int)
            {
                Log.i("xxx",  "onTextChanged für editFahrenheit")

                // endlose Rekursion vermeiden
                if (!binding.editFahrenheit.hasFocus())
                    return

                try {
                    val f = binding.editFahrenheit.text.toString().toDouble()
                    // siehe https://de.wikipedia.org/wiki/Grad_Fahrenheit
                    val c = (f - 32.0) / 1.8
                    binding.editCelsius.setText(round(c).toInt().toString())
                } catch (e: Exception) {
                    val t = Toast.makeText(applicationContext,
                        "Fehler: " + e.message, Toast.LENGTH_LONG)
                    t.setGravity(Gravity.CENTER, 0, 0)
                    t.show()
                    Log.e("xxx", "Fehler $e")
                    binding.editCelsius.setText("")
                }
            }
        })

        binding.editCelsius.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {}

            override fun beforeTextChanged(
                s: CharSequence?, start: Int, count: Int,
                after: Int) {  }

            override fun onTextChanged(s: CharSequence?,
                                       start: Int,
                                       before: Int,
                                       count: Int)
            {
                println("onTextChanged für editCelsius")

                // endlose Rekursion vermeiden
                if (!binding.editCelsius.hasFocus())
                    return

                try {
                    val c = binding.editCelsius.text.toString().toDouble()
                    val f = c * 1.8 + 32
                    binding.editFahrenheit.setText(round(f).toInt().toString())
                } catch (e: Exception) {
                    binding.editFahrenheit.setText("")
                }
            }
        })

        setContentView(binding.root)
    }
}
