package info.kofler.permissions

import android.Manifest
import android.content.pm.PackageManager
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.View.INVISIBLE
import android.view.View.VISIBLE
import androidx.core.app.ActivityCompat
import com.google.android.gms.location.*
import info.kofler.permissions.databinding.ActivityMainBinding
// import kotlinx.android.synthetic.main.activity_main.*


class MainActivity : AppCompatActivity() {
    private lateinit var binding: ActivityMainBinding

    // Quelle für den Code des Standortdienstes: https://stackoverflow.com/questions/45958226
    private lateinit var locClient: FusedLocationProviderClient
    private lateinit var locRequest: LocationRequest
    private lateinit var locCallback: LocationCallback


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        setupLocationHandler()

        // Zugriffsrechte klären
        val check = ActivityCompat.checkSelfPermission(
            this, Manifest.permission.ACCESS_FINE_LOCATION)
        if (check == PackageManager.PERMISSION_GRANTED) {
            // Zugriff erlaubt, Position ermitteln
            binding.lblInfo.visibility = INVISIBLE
            Log.i("xxx", "onCreate: granted")
        } else {
            // anfragen, ob Zugriff gewährt wird
            binding.lblInfo.visibility = VISIBLE
            requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), 123)
            Log.i("xxx", "onCreate: ask")
        }
    }

    // Ergebnis der Permission-Anfrage verarbeiten
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode != 123) return

        if (grantResults.isNotEmpty() && grantResults.first() == PackageManager.PERMISSION_GRANTED) {
            Log.i("xxx", "onReqPermResult: granted")
            binding.lblInfo.visibility = INVISIBLE
            startLocationUpdates()
        } else {
            Log.i("xxx", "onReqPermResult: not granted")
            binding.lblInfo.visibility = VISIBLE
        }
    }

     // Standortdienst deaktivieren, wenn die App nicht im Vordergrund ist
    override fun onPause() {
        super.onPause()
        stopLocationUpdates()
    }

    // Standortdienst wieder aktivieren
    override fun onResume() {
        super.onResume()
        startLocationUpdates()
    }

    // Standortdienst einrichten
    private fun setupLocationHandler() {
        locClient = LocationServices.getFusedLocationProviderClient(applicationContext)
        // locRequest = LocationRequest()
        locRequest = LocationRequest.create()
        locRequest.interval = 1000            // 1x pro Sekunde aktualisieren
        locRequest.fastestInterval = 1000
        locRequest.smallestDisplacement = 2f  // sofern 2m Unterschied
        locRequest.priority = LocationRequest.PRIORITY_HIGH_ACCURACY

        locCallback = object : LocationCallback() {
            override fun onLocationResult(locationResult: LocationResult?) {
                locationResult ?: return

                if (locationResult.locations.isNotEmpty()) {
                    val location = locationResult.lastLocation
                    binding.lblLocation.text = "Längengrad: ${location.latitude}\n" +
                        "Breitengrad: ${location.longitude}"
                    Log.i("xxx", "Time ${location.time}")
                }
            }
        }
    }

    //start location updates
    private fun startLocationUpdates() {
        val check = ActivityCompat.checkSelfPermission(
            this, Manifest.permission.ACCESS_FINE_LOCATION)
        if (check != PackageManager.PERMISSION_GRANTED) {
            Log.e("xxx", "startLocationUpdate: no permission")
            binding.lblInfo.visibility = VISIBLE
        } else {
            locClient.requestLocationUpdates(locRequest,
                locCallback, null)
            binding.lblInfo.visibility = INVISIBLE
        }
    }

    // stop location updates
    private fun stopLocationUpdates() {
        locClient.removeLocationUpdates(locCallback)
    }
}