package info.kofler.menu


import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.view.*
import androidx.fragment.app.Fragment
import android.widget.EditText
import android.widget.RelativeLayout
import android.widget.Switch
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.findNavController
import androidx.navigation.fragment.findNavController
import info.kofler.menu.databinding.FragmentF1Binding

class Fragment1 : Fragment() {

    private var _binding: FragmentF1Binding? = null
    // This property is only valid between onCreateView and
    // onDestroyView!
    private val binding get() = _binding!!
    
    private lateinit var mydata : MyData

    // wichtig: für edit1 + edit2 gilt saveEnabled = false!!!
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?): View?
    {
        Log.i("xxx", "onCreateView in Fragment1")
        // Log.i("xxx", "rnd2=" + MainActivity.rnd2.toString())

        mydata = ViewModelProvider(requireActivity()).get(MyData::class.java)
        _binding = FragmentF1Binding.inflate(inflater, container, false)
        
        // Menü aktivieren (erfordert auch onCreateOptionsMenu)
        setHasOptionsMenu(true)

        // ohne saveEnabled = false sind die edit-Anweisungen wirkungslos,
        // weil später der alte Status wiederhergestellt wird
        binding.edit1.setText(mydata.txt1.value)
        binding.edit2.setText(mydata.txt2.value)
        binding.txt3.text = mydata.txt3.value

        // bei jeder Änderung des Texts wird dieser
        // nach mydata übertragen
        val mtw1 = MyTextWatcher(binding.edit1, mydata.txt1)
        binding.edit1.addTextChangedListener(mtw1)
        val mtw2 = MyTextWatcher(binding.edit2, mydata.txt2)
        binding.edit2.addTextChangedListener(mtw2)

        // Navigation zum nächsten Fragment
        binding.btnNext.setOnClickListener {
            val navContr = binding.root.findNavController()
            navContr.navigate(R.id.action_fragment1_to_fragment2)
        }

        return binding.root
    }

    // Menü erzeugen
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        inflater.inflate(R.menu.menu1, menu)
        // val searchitem = menu.findItem(R.id.app_bar_search)!!
        // val searchview = searchitem.actionView

        val switchview = menu.findItem(R.id.app_bar_switch).actionView
        val layout = switchview.findViewById(R.id.app_bar_switch) as RelativeLayout?
        val switch = layout?.getChildAt(0) as? Switch
        switch?.setOnCheckedChangeListener { buttonView, isChecked ->
            Log.i("xxx", "App Bar Toggle State = $isChecked")
        }

    }

    // auf Menüauswahl reagieren
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        Log.i("xxx", "ItemSelected " + item.title.toString())

        if(item.itemId == R.id.menuSettings) {
            findNavController().navigate(R.id.action_fragment1_to_settingsFragment)
            return true
        } else if(item.itemId == R.id.menuAbout) {
            findNavController().navigate(R.id.action_fragment1_to_infoFragment)
            return true
        }
        return false
        // return super.onOptionsItemSelected(item)
    }

    //


}



// Verarbeitung von Eingaben in einem EditText, geänderter Text wird in Shared View Data gespeichert
class MyTextWatcher(val edit: EditText,
                    val livetxt: MutableLiveData<String>) : TextWatcher {

    // ignorieren
    override fun afterTextChanged(s: Editable?) { }

    // ignorieren
    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }

    // speichert aktuellen Text in einem
    // MutableLiveData-Objekt
    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
        livetxt.value = edit.text.toString()
        Log.i("xxx", "  txt=${livetxt.value}")
    }
}