package info.kofler.currencyconverter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.lifecycle.MutableLiveData
import androidx.recyclerview.widget.RecyclerView
import info.kofler.currencyconverter.databinding.ItemCurrencyBinding
import java.util.*

class CurrencyViewHolder(binding: ItemCurrencyBinding)
    : RecyclerView.ViewHolder(binding.root)
{
  val txtCurrency : TextView = binding.txtCurrency
  val imgFlag : ImageView = binding.imgFlag
}

// selectedCurrency: Zeichenkette mit der ausgewählten Währung
class CurrencyAdapter(private val cc: CurrencyCalculator,
                      private var selectedCurrency: MutableLiveData<String>,
                      private val context: Context)
  : RecyclerView.Adapter<CurrencyViewHolder>()
{
  // Farben definiert in res/values/colors.xml
  val colUnselected = ContextCompat.getColor(context,
    R.color.colorEntryBackground)
  val colSelected = ContextCompat.getColor(context,
    R.color.colorEntrySelectedBackground)

  // Anzahl der Elemente
  override fun getItemCount(): Int {
    return cc.rates.size
  }

  // Listenelement erzeugen
  override fun onCreateViewHolder(
    parent: ViewGroup, viewType: Int): CurrencyViewHolder
  {
    val binding = ItemCurrencyBinding.inflate(LayoutInflater.from(parent.context), parent, false)
    // Reaktion auf Auswahl
    binding.root.setOnClickListener {
      selectedCurrency.value = binding.txtCurrency.text.toString()
      notifyDataSetChanged()   // alles neu zeichnen
    }
    return CurrencyViewHolder(binding)
  }

  // Listenelement mit neuem Inhalt füllen
  override fun onBindViewHolder(
    holder: CurrencyViewHolder, position: Int)
  {
    val cur = cc.currencies[position]
    holder.txtCurrency.text = cur
    val imgid = getFlag(cur, context)
    holder.imgFlag.setImageResource(imgid)
    // unterschiedliche Hintergrund, falls ausgewählt
    if (cur == selectedCurrency.value) {
      holder.itemView.setBackgroundColor(colSelected)
    } else {
      holder.itemView.setBackgroundColor(colUnselected)
    }
  }
}

// Hilfsfunktion, ermittelt Resource-ID der Flagge
fun getFlag(currency: String, context: Context) : Int {
  if (currency.length < 2)
    return 0
  val cCode = currency.substring(0..1).lowercase(Locale.ROOT)
  val rscName = context.packageName + ":drawable/flag_$cCode"
  return context.resources.getIdentifier(rscName, null, null)
}