package info.kofler.currencyconverter

import android.annotation.SuppressLint
import android.content.Context
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.*
import org.w3c.dom.Element
import org.w3c.dom.NodeList
import java.io.File
import java.lang.Exception
import java.text.SimpleDateFormat
import java.util.*
import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.xpath.XPathConstants
import javax.xml.xpath.XPathFactory
import kotlin.text.Charsets.UTF_8

class CurrencyCalculator : ViewModel() {
    var isInitialized = false

    // Backup der Kurse speichern/laden
    private val dateftm = SimpleDateFormat("yyyy-MM-dd")

    // Datum der Kurse
    var lastupdate = Date(Long.MIN_VALUE)

    // unbedingt erforderlich, falls die Kurse mangels
    // Internetverbindung nicht geladen werden können
    var rates = mutableMapOf("EUR" to 1.0)

    // sortierte Liste der Währungen
    var currencies = listOf("EUR")

    // aktuell ausgewählte Kurse
    val cur1 = MutableLiveData<String>()
    val cur2 = MutableLiveData<String>()

    // EZB-Kurse laden; die Methode muss in einem
    // Hintergrund-Thread ausgeführt werden!
    @SuppressLint("SimpleDateFormat")
    fun getEcbRates(context: Context) : Boolean {
        try {
            val ecburl =
                "https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"
            val doc = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().parse(ecburl)

            // Zugriff auf das Cube-Element mit den Daten
            val xpathfactory = XPathFactory.newInstance()
            val path = xpathfactory.newXPath()
            val expr = path.compile("/Envelope/Cube/Cube")
            val result =
                expr.evaluate(doc, XPathConstants.NODESET) as NodeList

            // Datum extrahieren
            val mainnode = result.item(0)
            val ecbdate = mainnode.attributes.getNamedItem("time")
            lastupdate = dateftm.parse(ecbdate.nodeValue) ?: Date(0L)  // 1.1.1970 als Default

            // Schleife über alle Cube-Subitems mit den
            // Umrechnungsfaktoren für die diversen Währungen
            val currencyNodes = mainnode.childNodes
            for (i in 0 until currencyNodes.length) {
                val node = currencyNodes.item(i)
                if (node is Element) {
                    val cur =  node.getAttribute("currency")
                    val rate = node.getAttribute("rate")
                    rates[cur] = rate.toDouble()
                }
            }
            currencies = rates.keys.sorted()
            // Kurse speichern
            saveRates(context)
            return true
        } catch (e: Exception) {
            Log.e("xxx", e.message ?: "no message")
            // alte Version der Kurse laden
            loadRates(context)
        }
        return false
    }


    // lokales Backup der Umrechnungsfaktoren im Cache-
    // Verzeichnis speichern (JSON-Format)
    private fun saveRates(context: Context) {
        // nichts speichern, wenn keine echten Daten vorliegen
        if (rates.size <= 1)
            return

        try {
            val data = LastValidData(dateftm.format(lastupdate), rates)
            val jsonTxt = Json.encodeToString(LastValidData.serializer(), data)
            File(context.cacheDir, "backup.json").writeText(jsonTxt)
        } catch (e: Exception) {
            Log.e("xxx", "Fehler in saveRates: $e")
        }

    }

    // lokales Backup der Umrechnungsfaktoren laden (falls
    // gerade kein Internetzugang)
    private fun loadRates(context: Context) {
        val fileBackup = File(context.cacheDir, "backup.json")
        try {
            if (!fileBackup.isFile) return
            val txt = fileBackup.readText(UTF_8)
            val data = Json.decodeFromString(LastValidData.serializer(), txt)
            Log.i("xxx", "read $data")
            rates = data.rates.toMutableMap()
            lastupdate = dateftm.parse(data.update) ?: Date(0L)
        } catch (e: Exception) {
            Log.e("xxx", "Fehler in loadRates: $e")
        }
    }

    // Umrechnung von einer Währung in eine andere
    fun convert1to2(value: Double) : Double {
        val fromRate = rates[cur1.value]
        val toRate = rates[cur2.value]
        if (fromRate is Double && fromRate != 0.0 && toRate is Double) {
            return value / fromRate * toRate
        }
        return 0.0
    }

    fun convert2to1(value: Double) : Double {
        val fromRate = rates[cur2.value]
        val toRate = rates[cur1.value]
        if (fromRate is Double && fromRate != 0.0 && toRate is Double) {
            return value / fromRate * toRate
        }
        return 0.0
    }
}

@Serializable
class LastValidData(val update: String, val rates: Map<String, Double>)
