package info.kofler.currencyconverter

import android.content.Context
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.findNavController
import androidx.navigation.ui.NavigationUI
import info.kofler.currencyconverter.databinding.ActivityMainBinding
// import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.coroutines.*

class MainActivity : AppCompatActivity(), CoroutineScope by MainScope() {

    private lateinit var binding: ActivityMainBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        // Shared Data (View Model) für die ausgewählten
        // Währungen initialisieren (nur einmal)
        val cc = ViewModelProvider(this)
            .get(CurrencyCalculator::class.java)

        if (!cc.isInitialized) {
            // zuletzt gespeicherte Währungen lesen (wenn noch nie
            // gespeichert wurde: EUR + USD als Default)
            val pref = getPreferences(Context.MODE_PRIVATE)
            cc.cur1.value = pref.getString("CUR1",  "EUR")
            cc.cur2.value = pref.getString("CUR2",  "USD")

            // Observer für cur1/cur2 speichern Währungseinstellungen
            cc.cur1.observeForever {
                pref.edit().putString("CUR1", it).apply()
            }
            cc.cur2.observeForever {
                pref.edit().putString("CUR2", it).apply()
            }

            // Währungsdaten im Hintergrund mit Daten befüllen
            launch(Dispatchers.IO) {
                cc.getEcbRates(applicationContext)
            }

            cc.isInitialized = true
        }

        binding.fragHost.post {
            val navController = findNavController(R.id.fragHost)
            NavigationUI.setupActionBarWithNavController(this, navController)
        }


    }

    // im Navigation-Controller Up-Navigation aktivieren/durchführen
    override fun onSupportNavigateUp(): Boolean {
        val navController = this.findNavController(R.id.fragHost)
        return navController.navigateUp()
    }


     // eventuell noch im Hintergrund laufende Prozesse stoppen
    override fun onDestroy() {
        super.onDestroy()
        cancel()  // evt. noch im Hintergrund laufende Prozesse stoppen
    }

}
