package info.kofler.currencyconverter


import android.app.Activity
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.text.method.DigitsKeyListener
import android.view.*
import androidx.fragment.app.Fragment
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.findNavController
import androidx.navigation.ui.NavigationUI
import info.kofler.currencyconverter.databinding.FragmentMainBinding
// import kotlinx.android.synthetic.main.fragment_main.view.*
import java.text.DecimalFormatSymbols


class MainFragment : Fragment() {
    // layout/fragment_myname.xml -> FragmentMynameBinding usw.
    private var _binding: FragmentMainBinding? = null
    // This property is only valid between onCreateView and
    // onDestroyView!
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?): View?
    {
        // Fragment-Inhalt aus Layout-Datei erzeugen
        // val newfrag = inflater.inflate(R.layout.fragment_main, container, false)
        _binding = FragmentMainBinding.inflate(inflater, container, false)
        setHasOptionsMenu(true)  // Menü-Button anzeigen

        // Shared Data
        val cc = ViewModelProvider(requireActivity())
            .get(CurrencyCalculator::class.java)

        // Beschriftung und Flaggen
        binding.currency1.text = cc.cur1.value
        binding.currency2.text = cc.cur2.value
        binding.edit1.setText("0") // erfordert saveEnabled = false
        binding.edit2.setText("0")
        binding.flag1.setImageResource(
            getFlag(cc.cur1.value ?: "", requireContext()))
        binding.flag2.setImageResource(
            getFlag(cc.cur2.value ?: "", requireContext()))

        // Anklicken von Flagge/Währung -> Setup-Fragment
        val showSetup = {
            requireView().findNavController().navigate(
                R.id.action_mainFragment_to_settingsFragment)
        }
        binding.currency1.setOnClickListener { showSetup() }
        binding.currency2.setOnClickListener { showSetup() }
        binding.flag1.setOnClickListener     { showSetup() }
        binding.flag2.setOnClickListener     { showSetup() }

        // in D: Komma erlaubt, Punkt nicht
        // in US/GB: Punkt erlaubt, Komma nicht
        val sep = DecimalFormatSymbols.getInstance().decimalSeparator
        binding.edit1.keyListener =
            DigitsKeyListener.getInstance("0123456789$sep")
        binding.edit2.keyListener =
            DigitsKeyListener.getInstance("0123456789$sep")

        // Watcher kümmert sich um die Verarbeitung des Inputs
        // und die Umrechnung in die jeweils andere Währung
        val watcher1 = MyTextWatcher(binding.edit1, binding.edit2,
            cc, from1to2 = true)
        val watcher2 = MyTextWatcher(binding.edit2, binding.edit1,
            cc, from1to2 = false)
        binding.edit1.addTextChangedListener(watcher1)
        binding.edit2.addTextChangedListener(watcher2)

        // Touch im Fragment (außerhalb der Texteingabefelder)
        // blendet Tastatur aus
        binding.root.rootView.setOnClickListener {
            val imm =
                requireContext().getSystemService(Activity.INPUT_METHOD_SERVICE)
                        as InputMethodManager
            imm.hideSoftInputFromWindow(binding.root.windowToken, 0)
        }

        return binding.root
    }



    // Menü erzeugen
    override fun onCreateOptionsMenu(menu: Menu,
                                     inflater: MenuInflater)
    {
        super.onCreateOptionsMenu(menu, inflater)
        inflater.inflate(R.menu.menu, menu)
    }

    // auf Menüauswahl reagieren
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val navCtr = requireView().findNavController()
        return NavigationUI.onNavDestinationSelected(item, navCtr)
                || super.onOptionsItemSelected(item)
    }

}

// Verarbeitung der Texteingaben in den beiden EditText's
private class MyTextWatcher(val thisedit: EditText,
                            val otheredit: EditText,
                            val cc: CurrencyCalculator,
                            val from1to2: Boolean) : TextWatcher
{
    // ignorieren
    override fun afterTextChanged(s: Editable?) { }

    // ignorieren
    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) { }

    // jeweils anderes EditText-Feld aktualisieren
    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
        if (!thisedit.hasFocus())  // Rekursion vermeiden
            return

        // Texteingabe als Zahl interpretieren
        val value1 = thisedit.text.toString()
            .replace(",", ".").toDoubleOrNull() ?: 0.0

        // Umrechnung je nach Richtung
        if(from1to2) {
            val value2 = cc.convert1to2(value1)
            otheredit.setText("%.1f".format(value2))
        } else {
            val value2 = cc.convert2to1(value1)
            otheredit.setText("%.1f".format(value2))
        }
    }
}